/*
    Затвор Ловеч, София, Търново и Белене
    За Митко Мраморния, който сега е на почивка
*/

#include "gcd5.h"
#include <algorithm>
#include <iostream>
#include <numeric>
#include <cassert>
#include <vector>
#include <set>

typedef long long llong;
const int MAXN = 100000 + 10;
const llong INF = 1e18;
const int INTINF = 1e9;

namespace
{
    int n;  
}

struct Point
{
    llong x, y;
};

struct Line
{
    llong a;
    llong b;
    Line()
    {
        a = b = 0;
    }

    Line (Point s, Point e)
    {
        assert(s.x < e.x);
        assert(llabs(s.y - e.y) % (e.x - s.x) == 0);
        a = (e.y - s.y) / (e.x - s.x);
        b = s.y - s.x * a;
    }

    Line (llong _a, llong _b)
    {
        a = _a; b = _b;
    }

    llong at(llong x)
    {
        return a * x + b;
    }

    bool lies(Point p)
    {
        return p.y == p.x * a + b;
    }

    friend bool operator < (const Line &a, const Line &b)
    {
        return a.a < b.a || (a.a == b.a && a.b < b.b);
    }

    friend bool operator == (const Line &a, const Line &b)
    {
        return a.a == b.a && a.b == b.b;
    }

    friend bool operator != (const Line &a, const Line &b)
    {
        return !(a == b);
    }
};

llong getIntersectionX(Line l, Line r)
{
    if (l.at(-INTINF) > r.at(-INTINF))
    {
        std::swap(l, r);
    }

    llong ll = -INTINF, rr = INTINF, mid;
    while (ll < rr - 1)
    {
        mid = (ll + rr) / 2;
        if (l.at(mid) <= r.at(mid)) ll = mid;
        else rr = mid;
    }

    // std::cout << "intserc"
    return ll;
}

std::set <Line> s;
std::vector <Line> v;
void answer()
{
    for (const Line &l : v)
    {
        s.insert(l);
    }

    assert(s.size() >= n);
    for (const auto &[a, b] : s)
    {
        answer(a, b);
    }
}

void test()
{
    std::cout << "lines to now: " << v.size() << '\n';
    // assert(v.size() == n);
    for (const auto &[a, b] : v)
    {
        std::cout << a << ' ' << b << '\n';
        // answer(a, b);
    }

    std::cout << std::flush;
}

void rec(Line leftLine, Line rightLine)
{
    // std::cout << "call: " << leftLine.a << ' ' << leftLine.b << ' ' << rightLine.a << ' ' << rightLine.b << '\n';
    llong x = getIntersectionX(leftLine, rightLine);
    llong y = query(x);

    // std::cout << "intersection: " << x << ' ' << y << '\n';
    if (leftLine.lies({x, y}))
    {
        return;
    }
    
    llong nextY = query(x + 1);
    llong next2Y = query(x + 2);
    llong prevY = query(x - 1);
    Line lSuffix({x - 1, prevY}, {x, y});
    Line rPrefix({x + 1, nextY}, {x + 2, next2Y});
    
    // std::cout << "lines: " << lSuffix.a << ' ' << lSuffix.b << '\n';
    // std::cout << "lines: " << rPrefix.a << ' ' << rPrefix.b << '\n';
    if (rPrefix.lies({x, y}))
    {
        v.push_back(rPrefix);
        // assert(rPrefix != leftLine && rPrefix != rightLine);
        rec(leftLine, rPrefix);
        rec(rPrefix, rightLine);
    } else if (lSuffix.lies({x + 1, nextY}))
    {
        v.push_back(lSuffix);
        // assert(lSuffix != leftLine && lSuffix != rightLine);
        rec(leftLine, lSuffix);
        rec(lSuffix, rightLine);
    } else
    {
        if (lSuffix != leftLine)
        {
            v.push_back(lSuffix);
            rec(leftLine, lSuffix);
        }

        if (rPrefix != rightLine)
        {
            v.push_back(rPrefix);
            rec(rPrefix, rightLine);
        }
    }
}

void solve(int n) 
{
    ::n = n;
    Point lB = {-INTINF, query(-INTINF)}; 
    Point lE = {-INTINF + 1, query(-INTINF + 1)}; 
    v.push_back(Line(lB, lE));

    if (n == 1)
    {
        answer();
        return;
    }

    Point rB = {INTINF - 1, query(INTINF - 1)}; 
    Point rE = {INTINF, query(INTINF)}; 
    v.push_back(Line(rB, rE));

    // test();
    rec(v[0], v[1]);
    answer();
}
