/*
    Затвор Ловеч, София, Търново и Белене
    За Митко Мраморния, който сега е на почивка
*/

#include "gcd5.h"
#include <algorithm>
#include <iostream>
#include <numeric>
#include <cassert>
#include <vector>
#include <random>
#include <chrono>
#include <queue>
#include <set>

typedef long long llong;
const int MAXN = 100000 + 10;
const llong INF = 1e18;
const int INTINF = 1e9;

namespace
{
    int n;  
}

struct Point
{
    llong x, y;
};

struct Line
{
    llong a;
    llong b;
    Line()
    {
        a = b = 0;
    }

    Line (Point s, Point e)
    {
        assert(s.x < e.x);
        assert(llabs(s.y - e.y) % (e.x - s.x) == 0);
        a = (e.y - s.y) / (e.x - s.x);
        b = s.y - s.x * a;
    }

    Line (llong _a, llong _b)
    {
        a = _a; b = _b;
    }

    llong at(llong x)
    {
        return a * x + b;
    }

    bool lies(Point p)
    {
        return p.y == p.x * a + b;
    }

    friend bool operator < (const Line &a, const Line &b)
    {
        return a.a < b.a || (a.a == b.a && a.b < b.b);
    }

    friend bool operator == (const Line &a, const Line &b)
    {
        return a.a == b.a && a.b == b.b;
    }

    friend bool operator != (const Line &a, const Line &b)
    {
        return !(a == b);
    }
};

llong getIntersectionX(Line l, Line r)
{
    if (l.at(-INTINF) > r.at(-INTINF))
    {
        std::swap(l, r);
    }

    llong ll = -INTINF, rr = INTINF, mid;
    while (ll < rr - 1)
    {
        mid = (ll + rr) / 2;
        if (l.at(mid) <= r.at(mid)) ll = mid;
        else rr = mid;
    }

    // std::cout << "intserc"
    return ll;
}

bool goodIntersectionX(Line l, Line r)
{
    if (l.at(-INTINF) > r.at(-INTINF))
    {
        std::swap(l, r);
    }

    llong ll = -INTINF, rr = INTINF, mid;
    while (ll < rr - 1)
    {
        mid = (ll + rr) / 2;
        if (l.at(mid) <= r.at(mid)) ll = mid;
        else rr = mid;
    }

    // std::cout << "intserc"
    return {l.at(mid) == r.at(mid)};
}

std::set <Line> s;
std::vector <Line> v;
void answer()
{
    for (const auto &[a, b] : s)
    {
        answer(a, b);
    }
}

void test()
{
    std::cout << "lines to now: " << v.size() << '\n';
    // assert(v.size() == n);
    for (const auto &[a, b] : v)
    {
        std::cout << a << ' ' << b << '\n';
        // answer(a, b);
    }

    std::cout << std::flush;
}

std::mt19937 rng(std::chrono::steady_clock::now().time_since_epoch().count());
std::queue <std::pair <Line, Line>> q;
void rec(Line leftLine, Line rightLine)
{
    if (s.size() == n)
    {
        return;
    }

    // std::cout << "call: " << leftLine.a << ' ' << leftLine.b << ' ' << rightLine.a << ' ' << rightLine.b << '\n';
    llong x = getIntersectionX(leftLine, rightLine);
    llong y = query(x);

    if (goodIntersectionX(leftLine, rightLine) && leftLine.lies({x, y}))
    {
        return;
    }

    // std::cout << "intersection: " << x << ' ' << y << '\n';
    llong nextY = query(x + 1);
    if (leftLine.lies({x, y}) && rightLine.lies({x + 1, nextY}))
    {
        return;
    }

    if (leftLine.lies({x, y}))
    {
        llong next2Y = query(x + 2);
        if (rightLine.lies({x + 1, next2Y}))
        {
            return;
        }

        Line rPrefix({x + 1, nextY}, {x + 2, next2Y});
        s.insert(rPrefix);
        v.push_back(rPrefix);
        q.push({rPrefix, rightLine});
        return;
    }

    if (rightLine.lies({x + 1, nextY}))
    {
        llong prevY = query(x - 1);
        Line lSuffix({x - 1, prevY}, {x, y});
        s.insert(lSuffix);
        v.push_back(lSuffix);
        // assert(lSuffix != leftLine && lSuffix != rightLine);
        q.push({leftLine, lSuffix});
        q.push({lSuffix, rightLine});
        return;
    }
    
    llong prevY = query(x - 1);
    Line lSuffix({x - 1, prevY}, {x, y});
    if (lSuffix.lies({x + 1, nextY}))
    {
        s.insert(lSuffix);
        v.push_back(lSuffix);
        // assert(lSuffix != leftLine && lSuffix != rightLine);
        q.push({leftLine, lSuffix});
        q.push({lSuffix, rightLine});
        return;
    }

    llong next2Y = query(x + 2);
    Line rPrefix({x + 1, nextY}, {x + 2, next2Y});
    // std::cout << "lines: " << lSuffix.a << ' ' << lSuffix.b << '\n';
    // std::cout << "lines: " << rPrefix.a << ' ' << rPrefix.b << '\n';
    
    if (rPrefix.lies({x, y}))
    {
        llong prev2Y = query(x - 2);
        lSuffix = Line({x - 2, prev2Y}, {x - 1, prevY});
        s.insert(lSuffix);
        s.insert(rPrefix);

        if (lSuffix != leftLine)
        {
            v.push_back(lSuffix);
            q.push({leftLine, lSuffix});
        }

        if (rPrefix != rightLine)
        {
            v.push_back(rPrefix);
            q.push({rPrefix, rightLine});
        }
    } else
    {
        s.insert(lSuffix);
        s.insert(rPrefix);

        if (lSuffix != leftLine)
        {
            v.push_back(lSuffix);
            q.push({leftLine, lSuffix});
        }

        if (rPrefix != rightLine)
        {
            v.push_back(rPrefix);
            q.push({rPrefix, rightLine});
        }
    }
}

void run(Line a, Line b)
{
    q.push({a, b});
    while (q.size())
    {
        auto [l, r] = q.front();
        q.pop();

        rec(l, r);
    }
}

void solve(int n) 
{
    ::n = n;
    Point lB = {-INTINF, query(-INTINF)}; 
    Point lE = {-INTINF + 1, query(-INTINF + 1)}; 
    v.push_back(Line(lB, lE));
    s.insert(v[0]);

    if (n == 1)
    {
        answer();
        return;
    }

    Point rB = {INTINF - 1, query(INTINF - 1)}; 
    Point rE = {INTINF, query(INTINF)}; 
    v.push_back(Line(rB, rE));

    // test();
    run(v[0], v[1]);
    s.insert(v[1]);
    answer();
}