/*
    Затвор Ловеч, София, Търново и Белене
    За Митко Мраморния, който сега е на почивка
*/

#include <algorithm>
#include <iostream>
#include <numeric>
#include <cassert>
#include <vector>

typedef long long llong;
const int MAXN = 100000 + 10;
const llong INF = 1e18;
const int INTINF = 1e9;

int n, q;
struct SegmentTree
{
    struct Node
    {
        llong value;
        llong lazy;
        int maxIdx;

        Node()
        {
            value = lazy = maxIdx = 0;
        }

        friend Node operator + (const Node &left, const Node &right)
        {
            if (left.value > right.value)
            {
                return left;
            } else
            {
                return right;
            }
        }
    };

    Node tree[4*MAXN];
    void build(int l, int r, int node, std::vector <llong> &dists)
    {
        tree[node] = Node();
        if (l == r)
        {
            tree[node].value = dists[l];
            tree[node].maxIdx = l;
            return;
        }

        int mid = l + r >> 1;
        build(l, mid, 2*node, dists);
        build(mid + 1, r, 2*node + 1, dists);
        tree[node] = tree[2*node] + tree[2*node + 1];
    }

    void push(int node, int l, int r)
    {
        if (tree[node].lazy == 0)
        {
            return;
        }

        tree[node].value -= tree[node].lazy;
        if (l < r)
        {
            tree[2*node].lazy += tree[node].lazy;
            tree[2*node + 1].lazy += tree[node].lazy;
        }

        tree[node].lazy = 0;
    }

    void update(int l, int r, int node, int queryL, int queryR, llong queryVal)
    {
        push(node, l, r);
        if (queryR < l || r < queryL)
        {
            return;
        }

        if (queryL <= l && r <= queryR)
        {
            tree[node].lazy += queryVal;
            push(node, l, r);
            return;
        }

        int mid = l + r >> 1;
        update(l, mid, 2*node, queryL, queryR, queryVal);
        update(mid + 1, r, 2*node + 1, queryL, queryR, queryVal);
        tree[node] = tree[2*node] + tree[2*node + 1];
    }

    void build(std::vector <llong> &tour)
    {
        build(0, n - 1, 1, tour);
    }

    void update(int l, int r, int val)
    {
        update(0, n - 1, 1, l, r, val);
    }

    Node getMAX()
    {
        return tree[1];
    }
};

int in[MAXN];
int out[MAXN];
int par[MAXN];
llong dist[MAXN];
int parEdgeIdx[MAXN];
int parEdgeVal[MAXN];
llong answerFor[MAXN];
std::vector <int> tour;
std::vector <llong> dists;
std::vector <std::pair <int,int>> g[MAXN];
SegmentTree tree;

void reset()
{
    for (int i = 1 ; i <= n ; ++i)
    {
        g[i].clear();
    }
}

void calcDist(int node, int p)
{
    int cnt = 0;
    par[node] = p;
    in[node] = tour.size();
    tour.push_back(node);
    dists.push_back(dist[node]);

    for (const auto &[u, w] : g[node])
    {
        cnt++;
        if (u == p)
        {
            continue;
        }

        dist[u] = dist[node] + w;
        parEdgeIdx[u] = cnt - 1;
        parEdgeVal[u] = w;
        calcDist(u, node);
    }

    out[node] = tour.size() - 1;
}

void clearDist(int node)
{
    if (node != 1)
    {
        if (g[par[node]][parEdgeIdx[node]].second == 0)
        {
            return;
        }

        g[par[node]][parEdgeIdx[node]].second = 0;
        tree.update(in[node], out[node], parEdgeVal[node]);
        clearDist(par[node]);
    }
}

llong sumEdges;
llong calcFor(int people, llong p)
{
    return 2 * sumEdges - answerFor[people] + (people - 1) * p;
}

void solve()
{
    tour.clear();
    dists.clear();
    calcDist(1, 0);
    llong sumDist = 0;
    tree.build(dists);

    for (int i = 1 ; i <= n ; ++i)
    {
        SegmentTree::Node res = tree.getMAX();
        if (res.value != 0)
        {
            sumDist += res.value;
            clearDist(tour[res.maxIdx]);
        }

        answerFor[i] = sumDist;
    }

    for (int i = 1 ; i <= q ; ++i)
    {
        llong p;
        std::cin >> p;

        int l = 0, r = n, mid;
        while (l < r - 1)
        {
            mid = (l + r) / 2;
            if (calcFor(mid, p) - calcFor(mid + 1, p) >= 0) l = mid;
            else r = mid;
        }

        std::cout << calcFor(r, p) << '\n';
    }
}

void input()
{
    sumEdges = 0;
    std::cin >> n;
    for (int i = 1 ; i < n ; ++i)
    {
        int u, v, d;
        std::cin >> u >> v >> d;
        g[u].push_back({v, d});
        g[v].push_back({u, d});
        sumEdges += d;
    }

    std::cin >> q;
}

void fastIOI()
{
    std::ios_base :: sync_with_stdio(0);
    std::cout.tie(nullptr);
    std::cin.tie(nullptr);
}

int tests;
int main()
{
    fastIOI();
    std::cin >> tests;
    
    while (tests--)
    {
        reset();
        input();
        solve();
    }

    return 0;
}