#include <iostream>
#include <vector>
#include <utility>
#include <algorithm>

#define endl '\n'
#define int int64_t

using namespace std;

const int MAXN = 1e5 + 3;

vector<pair<int, int>> graph[MAXN];

vector<pair<int, int>> eulerTour;

bool compare(const pair<int, int> &p1, const pair<int, int> &p2) {
    return p1.second < p2.second;
}

void dfs(int currentNode, int parentNode, int parentDist) {
    eulerTour.push_back({currentNode, parentDist});

    sort(graph[currentNode].begin(), graph[currentNode].end());

    for (auto [nextNode, currentDist] : graph[currentNode]) {
        if (nextNode == parentNode) {
            continue;
        }

        dfs(nextNode, currentNode, currentDist);

        eulerTour.push_back({currentNode, currentDist});
    }
}

int64_t find_min_cost(int n, int64_t friendPrice) {
    vector<bool> isPainted(n, false);

    int64_t result = 0ll;
    int64_t currentSum = 0ll;

    for (auto [node, dist]: eulerTour) {
        if (isPainted[node]) {
            currentSum += 1ll * dist;
            continue;
        }

        if (currentSum > friendPrice) {
            currentSum = friendPrice;
        }

        result += currentSum;
        currentSum = 0ll;

        result += 1ll * dist;
        isPainted[node] = true;
    }

    return result;
}

void solve(int test) {
    int n;
    cin >> n;

    for (int i = 0; i < n; ++i) {
        graph[i].clear();
    }
    eulerTour.clear();

    for (int i = 1; i < n; ++i) {
        int from, to, dist;
        cin >> from >> to >> dist;

        --from;
        --to;

        graph[from].push_back({to, dist});
        graph[to].push_back({from, dist});
    }

    dfs(0, -1, 0);

    int q;
    cin >> q;

    for (int i = 0; i < q; ++i) {
        int friendPrice;
        cin >> friendPrice;

        cout << find_min_cost(n, 1ll * friendPrice) << endl;
    }
}

signed main() {
    ios::sync_with_stdio(false);
    cin.tie(nullptr);

    int tests;
    cin >> tests;

    for (int test = 0; test < tests; ++test) {
        solve(test);
    }
}
