#include <iostream>
#include <vector>
#include <utility>
#include <algorithm>

#define endl '\n'

using namespace std;

const int MAXN = 1e5 + 3;

vector<pair<int, int>> graph[MAXN];

vector<pair<int, int>> eulerTour;

int parentDist[MAXN];
int nodeGrade[MAXN];

bool customCompare(const pair<int, int> &p1, const pair<int, int> &p2) {
    return nodeGrade[p1.first] < nodeGrade[p2.first];
}

void dfs_grades(int currentNode, int parentNode) {
    nodeGrade[currentNode] = parentDist[currentNode];

    for (auto [nextNode, currentDist] : graph[currentNode]) {
        if (nextNode == parentNode) {
            continue;
        }

        parentDist[nextNode] = currentDist;
        dfs_grades(nextNode, currentNode);

        nodeGrade[currentNode] += nodeGrade[nextNode];
    }
}

void dfs_euler_tour(int currentNode, int parentNode) {
    eulerTour.push_back({currentNode, parentDist[currentNode]});

    sort(graph[currentNode].begin(), graph[currentNode].end(), customCompare);

    for (auto [nextNode, currentDist] : graph[currentNode]) {
        if (nextNode == parentNode) {
            continue;
        }

        dfs_euler_tour(nextNode, currentNode);

        eulerTour.push_back({currentNode, currentDist});
    }
}

int64_t find_min_cost(int n, int64_t friendPrice) {
    vector<bool> isPainted(n, false);

    int64_t result = 0ll;
    int64_t currentSum = 0ll;

    for (auto [node, dist]: eulerTour) {
        // cout << "(" << node + 1 << ", " << dist << ")\n";

        if (isPainted[node]) {
            currentSum += 1ll * dist;
            continue;
        }

        if (currentSum > friendPrice) {
            currentSum = friendPrice;
        }

        result += currentSum;
        currentSum = 0ll;

        result += 1ll * dist;
        isPainted[node] = true;
    }

    return result;
}

void solve(int test) {
    int n;
    cin >> n;

    for (int i = 0; i < n; ++i) {
        graph[i].clear();
        parentDist[i] = 0;
        nodeGrade[i] = 0;
    }
    eulerTour.clear();

    for (int i = 1; i < n; ++i) {
        int from, to, dist;
        cin >> from >> to >> dist;

        --from;
        --to;

        graph[from].push_back({to, dist});
        graph[to].push_back({from, dist});
    }

    dfs_grades(0, -1);

    dfs_euler_tour(0, -1);

    int q;
    cin >> q;

    for (int i = 0; i < q; ++i) {
        int friendPrice;
        cin >> friendPrice;

        cout << find_min_cost(n, 1ll * friendPrice) << endl;
    }
}

int main() {
    ios::sync_with_stdio(false);
    cin.tie(nullptr);

    int tests;
    cin >> tests;

    for (int test = 0; test < tests; ++test) {
        solve(test);
    }
}
