#include <algorithm>
#include <array>
#include <cmath>
#include <iostream>
#include <random>
#include <utility>
#include <vector>

#include"gcd5.cpp"

static std::vector<std::pair<long long, long long>> input_lines;
static std::vector<std::pair<long long, long long>> output;
static int Q = 0;

const int BOUND = (int)1e9;
const int Q_LIMIT = 5000000;

void check_value(int x) {
    if(x < -BOUND || x > BOUND) {
        std::cerr << "Value outside of valid range: " << x << std::endl;
        std::cout << -3 << std::endl;
        exit(0);
    }
}

void answer(long long a, long long b) {
    if(output.size() == input_lines.size()) {
        std::cerr << "Too many lines" << std::endl;
        std::cout << -2 << std::endl;
        exit(0);
    }

    output.push_back({a, b});
}

long long query(int x) {
    check_value(x);
    Q++;
    if(Q > Q_LIMIT) {
        std::cerr << "Too many queries" << std::endl;
        std::cout << -1 << std::endl;
        exit(0);
    }

    long long max_y = std::numeric_limits<long long>::min();
    for(auto line: input_lines) {
        max_y = std::max(max_y, line.first * 1ll * x + line.second);
    }

    return max_y;
}

int main() {
    int n;
    std::cin >> n;

    for(int i = 0; i < n; i++) {
        long long a, b;
        std::cin >> a >> b;
        input_lines.push_back({a, b});
    }
   /*) for (int i=0 ; i<input_lines.size() ; i++)
    {
        cout << input_lines[i].first << " " << input_lines[i].second << endl;
    }
    cout << endl;*/

    solve(n);

    std::sort(input_lines.begin(), input_lines.end());
    std::sort(output.begin(), output.end());

    if(input_lines != output) {
        std::cerr << "Wrong answer" << std::endl;
        std::cout << 0 << std::endl;
    } else {
        std::cerr << "Correct answer" << std::endl;
        std::cout << 1 << " " << Q << std::endl;
    }

    return 0;
}
